using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ClientUdp
{
    class ClientUdp
    {

        private static Socket clientSocket;
        private static EndPoint serverEP;
        public enum ClientStatus{ ENVOI, RECEPTION, STOP };
        private static ClientStatus status = ClientStatus.ENVOI;


        public static void Main(string[] args)
        {
            //variable de test des dépassements de mémoire.
            String bufferOverflow = new string('f', 10000);

            //Liste des test d'envoi de messages à faire
            Dictionary<string, object[]> listeTests = new Dictionary<string, object[]>();
            listeTests.Add("dataOverflow", new object[] { Commande.POST, CommandeType.REQUETE, "Tim", bufferOverflow });
            listeTests.Add("pseudoOverflow", new object[] { Commande.POST, CommandeType.REQUETE, bufferOverflow, "test overflow 2" });
            listeTests.Add("testPost", new object[] { Commande.POST, CommandeType.REQUETE, "Tim", "test post" });
            listeTests.Add("testSubscribe", new object[] { Commande.SUBSCRIBE, CommandeType.REQUETE, "Tim", "test subscribe" });
            listeTests.Add("testPost2", new object[] { Commande.POST, CommandeType.REQUETE, "Tim", "test post2" });
            listeTests.Add("testGet", new object[] { Commande.GET, CommandeType.REQUETE, "Tim", "test get" });
            

            Thread reception = new Thread(new ThreadStart(ReceptionMessage));

            try
            {
                //************************************************************** Initialisation
                string serverIP = "192.168.43.174";
                int serverPort = 11111;                 // A changer


                // Création de la socket d'écoute UDP
                clientSocket = new Socket(
                    //domaine (IPv4)
                    AddressFamily.InterNetwork,
                    //type de socket (Dgram = pas de connexions préalable)
                    SocketType.Dgram,
                    ProtocolType.Udp);


                // Liaison de la socket au point de communication
                clientSocket.Bind(new IPEndPoint(IPAddress.Any, 0));


                // Création du EndPoint serveur
                serverEP = new IPEndPoint(IPAddress.Parse(serverIP), serverPort);

                reception.Start();

                //envoi des messages
                foreach (var element in listeTests)
                {
                    if(status != ClientStatus.STOP)
                    {
                        object[] cmdParam = element.Value;
                        try
                        {
                            ChatMessage chatM = new ChatMessage((Commande)cmdParam[0], (CommandeType)cmdParam[1], (string)cmdParam[2], (string)cmdParam[3]);

                            // Encodage du string dans un buffer de bytes en ASCII
                            byte[] buffer = chatM.GetBytes();

                            // Envoie du message au serveur
                            int nBytes = clientSocket.SendTo(buffer, 0, buffer.Length, SocketFlags.None, serverEP);
                            PrintEnvoiMessage(chatM, element.Key, nBytes);
                        }
                        catch (ArgumentOutOfRangeException ex)
                        {
                            Console.WriteLine("");
                            Console.WriteLine("Erreur: " + ex.Message);
                        }
                    }
                    else
                    {
                        break;
                    }
                    
                }
                Console.WriteLine("Appuyez sur une touche pour quitter...");
                Console.ReadKey();
            }
            catch (SocketException E)
            {
                Console.WriteLine(E.Message);
            }
            finally
            {//En cas d'erreur ou à la fin, on ferme la socket et on arrete le thread de reception
                reception.Abort();
                Console.WriteLine("");
                Console.WriteLine("Fermeture Socket");
                clientSocket.Close();
            }
            Console.ReadKey();
        }

        public static void ReceptionMessage()
        {
            byte[] buffer = new byte[ChatMessage.bufferSize];

            while (status != ClientStatus.STOP)
            {
                try
                {
                    //On récupère le packet et on traite le message
                    int nBytes = clientSocket.ReceiveFrom(buffer, ChatMessage.bufferSize, SocketFlags.None, ref serverEP);
                    ChatMessage msg = new ChatMessage(buffer);
                    switch (msg.commande)
                    {
                        case Commande.POST:
                            PrintReceptionMessage(msg, nBytes);
                            break;
                        case Commande.GET:
                            PrintReceptionMessage(msg, nBytes);
                            break;
                        case Commande.HELP:
                            break;
                        case Commande.QUIT:
                            break;
                        case Commande.STOPSERVEUR:
                            PrintReceptionMessage(msg, nBytes);
                            status = ClientStatus.STOP;
                            break;
                        case Commande.SUBSCRIBE:
                            PrintReceptionMessage(msg, nBytes);
                            break;
                        case Commande.SUBSCRIBEv2:
                            break;
                        case Commande.UNSUBSCRIBE:
                            break;
                    }
                }
                catch (SocketException E)
                {
                    Console.WriteLine(E.Message);
                    status = ClientStatus.STOP;
                }
            }
        }

        public static void PrintEnvoiMessage(ChatMessage c, string nomMessage, int nBytes)
        {
            PrintMessage(c, "(" + nomMessage + ") ", "envoye vers ", nBytes);
        }
        
        public static void PrintReceptionMessage(ChatMessage c, int nBytes)
        {
            PrintMessage(c, "reçu de ", nBytes);
        }

        public static void PrintMessage(ChatMessage c, string action, int nBytes)
        {
            PrintMessage(c, "", action, nBytes);
        }

        public static void PrintMessage(ChatMessage c, string nomMessage, string action, int nBytes)
        {
            Console.WriteLine("");
            Console.Write("Nouveau message " + nomMessage);
            Console.ForegroundColor = ConsoleColor.Green;
            Console.Write(action);
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine(serverEP
                         + " (" + nBytes + " octets)"
                         + ": \"" + c + "\"");
        }

    }
}
