﻿using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ClientUdp;

namespace ServeurUdp
{
    class ServeurUdp
    {
        
        static List<ChatMessage> listeMessages = new List<ChatMessage>();

        //Liste d'EndPoint dans laquelle les personnes inscrites sont stockées sous la forme: <EndPoint,pseudo>
        static Dictionary<EndPoint, string> listeAbonnes = new Dictionary<EndPoint, string>();
        
        /* Dictionnaire représentant toutes les rooms, et comprenant: 
             - en clé le nom de la room
             - en valeur la liste des ChatMessages reçus par POST dans cette room.
        */    
        static Dictionary<string, List<ChatMessage>> listeRooms = new Dictionary<string, List<ChatMessage>>();

        static Socket serverSocket;
        static EndPoint clientEP;


        static void Main(string[] args)
        {

            try
            {
                // ************************************************************** Initialisation

                // Création de la socket d'écoute UDP
                serverSocket = new Socket(
                    AddressFamily.InterNetwork,
                    SocketType.Dgram,
                    ProtocolType.Udp);


                // Liaison de la socket au point de communication
                serverSocket.Bind(new IPEndPoint(IPAddress.Any, 11111));

                Boolean run = true;
                while (run)
                {
                    //************************************************************** Communications
                    Console.WriteLine("");
                    Console.WriteLine("Attente d'une nouveau message...");

                    // Reception message client
                    clientEP = new IPEndPoint(IPAddress.Any, 0);
                    byte[] buffer = new byte[1500];

                    int nBytes = serverSocket.ReceiveFrom(buffer, buffer.Length, SocketFlags.None, ref clientEP);
                    ChatMessage chatMsg = null;
                    try
                    {
                        chatMsg = new ChatMessage(buffer);
                    }
                    catch(ArgumentException e)
                    {
                        Console.WriteLine("Erreur : " + e.Message);
                    }


                    Console.WriteLine("Nouveau message de "
                        + clientEP
                        + " (" + nBytes + " octets)"
                        + ": \"" + chatMsg + "\"");

                    if(chatMsg != null)
                    {
                        //Regarde quelle méthode est envoyée pour envoyée le message
                        switch (chatMsg.commande)
                        {
                            //Méthode POST
                            case Commande.POST:
                                StockerMessage(chatMsg);
                                break;
                            //Méthode GET
                            case Commande.GET:
                                RecevoirMessage(chatMsg);
                                break;
                            //Méthode HELP
                            case Commande.HELP:
                                break;

                            //Méthode QUIT
                            case Commande.QUIT:
                                break;

                            //Méthode STOPSERVEUR
                            case Commande.STOPSERVEUR:
                                run = false;
                                break;

                            //Méthode SUBSCRIBE
                            case Commande.SUBSCRIBE:
                                SubscribeClient(chatMsg, clientEP);
                                break;

                            //Méthode SUBSCRIBEv2
                            case Commande.SUBSCRIBEv2:
                                break;

                            //Méthode UNSUBSCRIBE
                            case Commande.UNSUBSCRIBE:
                                break;
                            /*case Commande.CREATEROOM:
                                //CreateRoom(chatMsg);
                                break;
                            case Commande.LISTROOMS:
                                //ListerRoom();
                                break;*/
                        }
                    }
                }
            }

            catch (SocketException E)
            {
                Console.WriteLine(E.Message);
            }
            finally
            {//En cas d'erreur ou à la fin, on ferme la socket et on arrete le thread de reception
                Console.WriteLine("");
                Console.WriteLine("Fermeture Socket");
                serverSocket.Close();
                Console.WriteLine("Appuyez sur une touche pour quitter...");
                Console.ReadKey();
            }
        }
        public static void StockerMessage(ChatMessage c)
        {
            listeMessages.Add(c);
            Console.WriteLine("Le message '" + c.ToString() + "a été stocké");

            foreach (KeyValuePair<EndPoint, String> abonne in listeAbonnes)
            {
                serverSocket.SendTo(ChatMessage.GetBytes(Commande.SUBSCRIBE, CommandeType.REPONSE, c.pseudo, c.data), abonne.Key);
            }

        }

        public static void RecevoirMessage(ChatMessage c)
        {
            foreach (ChatMessage message in listeMessages)
            {
                serverSocket.SendTo(ChatMessage.GetBytes(Commande.GET, CommandeType.REPONSE, message.pseudo, message.data), clientEP);
            }

        }

        public static void SubscribeClient(ChatMessage c,EndPoint ep)
        {
            if (!listeAbonnes.ContainsKey(ep))
            {
                listeAbonnes.Add(ep, c.pseudo);
                Console.WriteLine(c.pseudo+" vient de s'abonner");
            }
            else
                Console.WriteLine("Le client est déjà enregistré pour recevoir les messages");
        }

        public static void UnsubscribeClient(ChatMessage c, EndPoint clientEP)
        {
            if (listeAbonnes.ContainsKey(clientEP))
            {
                listeAbonnes.Remove(clientEP);
                Console.WriteLine(c.pseudo + " vient de se desabonner");
            }
            else
            {
                Console.WriteLine("Le client n'est pas enregistré pour recevoir de messages");
            }
        }
        
        public static void CreateRoom(ChatMessage c)
        {
            if (!listeRooms.ContainsKey(c.data)){
                List<ChatMessage> lMessages = new List<ChatMessage>();
                listeRooms.Add(c.data, lMessages);
                Console.WriteLine("la room "+c.data + " vient d'etre creer");
            }
            else
                Console.WriteLine("La room existe déjà");
        }

    }
}