﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;

namespace TronClient
{
    public class Client
    {
        private Tron.Tron myTron;        // Moteur du jeu

        public byte frequence;      // Temps du tour de jeu (en dixieme de s)

        private Socket clientSocket;
        private IPEndPoint serverEP;



        // constructeur : IP/Port du serveur
        public Client(String myServerIP, int myServerPort)
        {
            // Creation de la socket d'écoute TCP
            clientSocket = new Socket(
                    //domaine (IPv4)
                    AddressFamily.InterNetwork,
                    //type de socket (Stream = connecté)
                    SocketType.Stream,
                    ProtocolType.Tcp);

            // Liaison de la socket au point de communication
            clientSocket.Bind(new IPEndPoint(IPAddress.Any, 0));


            // Création du EndPoint serveur
            serverEP = new IPEndPoint(IPAddress.Parse(myServerIP), myServerPort);
        }


        // Appelé au début de la partie
        public Tron.Tron Init()
        {
            System.Console.WriteLine("Init");

            //Connexion au serveur
            try
            {
                clientSocket.Connect(serverEP);
            } catch(Exception ex)
            {
                Console.WriteLine("impossible de se connecter au serveur, Error: " + ex.Message);
                clientSocket.Close();
                System.Environment.Exit(1);
            }
            


            // Réception des paramètres
            byte[] buffer = new byte[4];
            try
            {
                clientSocket.Receive(buffer, buffer.Length, SocketFlags.None);
            }catch(SocketException sEx)
            {
                Console.WriteLine(sEx.Message);
            }
            


            // Initialisation de la fréquence : frequence = <frequence>
            frequence = buffer[3];

            // Initialisation du moteur : myTron = new Tron(byte <taille terrain>, byte <nombre de joueurs>, byte <numéro du joueur>);
            myTron = new Tron.Tron(buffer[0], buffer[1], buffer[2]);

            // Retourne le moteur
            return myTron;
        }

        // Appelé régulièrement à chaque tour de jeu
        public void Routine()
        {
            System.Console.WriteLine("Routine");

            // Envoie de sa direction : myTron.getDirection()
            byte[] direction = new byte[1];
            direction[0] = myTron.getDirection();
            clientSocket.Send(direction, 0, direction.Length, SocketFlags.None);

            // Reception de toutes les directions : myTron.setDirections(byte[] < toutes les directions>);
            byte[] directions = new byte[myTron.getNJoueurs()];
            clientSocket.Receive(directions, 0, directions.Length, SocketFlags.None);
            myTron.setDirections(directions);


        }

        // Appelé à la fin de la partie
        public void Conclusion()
        {
            System.Console.WriteLine("Conclusion");

            // fermeture socket
            clientSocket.Close();
        }


        // propriété frequence (Ne pas toucher)
        public int freq
        {
            get { return frequence * 100; }
            set { }
        }
    }
}
